(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('react-dom'), require('prop-types'), require('a11y-dialog')) :
    typeof define === 'function' && define.amd ? define(['exports', 'react', 'react-dom', 'prop-types', 'a11y-dialog'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.A11yDialog = {}, global.React, global.ReactDOM, global.PropTypes, global.A11yDialogLib));
})(this, (function (exports, React, ReactDOM, PropTypes, A11yDialogLib) { 'use strict';

    function _interopNamespaceDefault(e) {
        var n = Object.create(null);
        if (e) {
            Object.keys(e).forEach(function (k) {
                if (k !== 'default') {
                    var d = Object.getOwnPropertyDescriptor(e, k);
                    Object.defineProperty(n, k, d.get ? d : {
                        enumerable: true,
                        get: function () { return e[k]; }
                    });
                }
            });
        }
        n.default = e;
        return Object.freeze(n);
    }

    var React__namespace = /*#__PURE__*/_interopNamespaceDefault(React);
    var ReactDOM__namespace = /*#__PURE__*/_interopNamespaceDefault(ReactDOM);

    const useIsMounted = () => {
        const [isMounted, setIsMounted] = React__namespace.useState(false);
        React__namespace.useEffect(() => setIsMounted(true), []);
        return isMounted;
    };
    const useA11yDialogInstance = () => {
        const [instance, setInstance] = React__namespace.useState(null);
        const container = React__namespace.useCallback((node) => {
            if (node !== null)
                setInstance(new A11yDialogLib(node));
        }, []);
        return [instance, container];
    };
    const useA11yDialog = (props) => {
        const [instance, ref] = useA11yDialogInstance();
        const close = React__namespace.useCallback(() => instance && instance.hide(), [instance]);
        const role = props.role || 'dialog';
        const isAlertDialog = role === 'alertdialog';
        const titleId = props.titleId || props.id + '-title';
        // Destroy the `a11y-dialog` instance when unmounting the component.
        React__namespace.useEffect(() => {
            return () => {
                if (instance)
                    instance.destroy();
            };
        }, [instance]);
        return [
            instance,
            {
                container: {
                    id: props.id,
                    ref,
                    role,
                    tabIndex: -1,
                    'aria-modal': true,
                    'aria-hidden': true,
                    'aria-labelledby': titleId,
                },
                overlay: { onClick: isAlertDialog ? undefined : close },
                dialog: { role: 'document' },
                closeButton: { type: 'button', onClick: close },
                // Using a paragraph with accessibility mapping can be useful to work
                // around SEO concerns of having multiple <h1> per page.
                // See: https://twitter.com/goetsu/status/1261253532315004930
                title: { role: 'heading', 'aria-level': 1, id: titleId },
            },
        ];
    };
    const A11yDialog = props => {
        if (!props.classNames)
            props.classNames = {};
        if (!props.dialogRef)
            props.dialogRef = () => [];
        const isMounted = useIsMounted();
        const [instance, attributes] = useA11yDialog(props);
        const { dialogRef } = props;
        React__namespace.useEffect(() => {
            if (instance)
                dialogRef(instance);
            return () => {
                dialogRef(undefined);
            };
        }, [dialogRef, instance]);
        if (!isMounted)
            return null;
        const root = props.dialogRoot
            ? document.querySelector(props.dialogRoot)
            : document.body;
        const title = (React__namespace.createElement("p", { ...attributes.title, className: props.classNames.title, key: 'title' }, props.title));
        const button = (React__namespace.createElement("button", { ...attributes.closeButton, className: props.classNames.closeButton, "aria-label": props.closeButtonLabel, key: 'button' }, props.closeButtonContent));
        const children = [
            props.closeButtonPosition === 'first' && button,
            title,
            props.children,
            props.closeButtonPosition === 'last' && button,
        ].filter(Boolean);
        return ReactDOM__namespace.createPortal(React__namespace.createElement("div", { ...attributes.container, className: props.classNames.container },
            React__namespace.createElement("div", { ...attributes.overlay, className: props.classNames.overlay }),
            React__namespace.createElement("div", { ...attributes.dialog, className: props.classNames.dialog }, children)), root);
    };
    A11yDialog.defaultProps = {
        role: 'dialog',
        closeButtonLabel: 'Close this dialog window',
        closeButtonContent: '\u00D7',
        closeButtonPosition: 'first',
        classNames: {},
        dialogRef: () => void 0,
        // Default properties cannot be based on other properties, so the default
        // value for the `titleId` prop is defined in the `render(..)` method.
    };
    A11yDialog.propTypes = {
        // The `role` attribute of the dialog element, either `dialog` (default) or
        // `alertdialog` to make it a modal (preventing closing on click outside of
        // ESC key).
        role: PropTypes.oneOf(['dialog', 'alertdialog']),
        // The HTML `id` attribute of the dialog element, internally used by
        // a11y-dialog to manipulate the dialog.
        id: PropTypes.string.isRequired,
        // The title of the dialog, mandatory in the document to provide context to
        // assistive technology. Could be hidden (while remaining accessible) with
        // CSS though.
        title: PropTypes.node.isRequired,
        // A function called when the component has mounted, receiving the instance
        // of A11yDialog so that it can be programmatically accessed later on.
        // E.g.: dialogRef={(dialog) => (this.dialog = dialog)}
        dialogRef: PropTypes.func,
        // The HTML `id` attribute of the dialog’s title element, used by assistive
        // technologies to provide context and meaning to the dialog window. Falls
        // back to the `${this.props.id}-title` if not provided.
        titleId: PropTypes.string,
        // The HTML `aria-label` attribute of the close button, used by assistive
        // technologies to provide extra meaning to the usual cross-mark. Defaults
        // to a generic English explanation.
        closeButtonLabel: PropTypes.string,
        // The string that is the innerHTML of the close button.
        closeButtonContent: PropTypes.node,
        // Whether the close button should be rendered as first/last children or not at all.
        closeButtonPosition: PropTypes.oneOf(['first', 'last', 'none']),
        // React 16 requires a container for the portal’s content to be rendered
        // into; this needs to be an existing valid DOM node and defaults to the body
        // element.
        dialogRoot: PropTypes.string,
        // Object of classes for each HTML element of the dialog element.
        // See: https://a11y-dialog.netlify.app/usage/markup
        classNames: PropTypes.shape({
            container: PropTypes.string.isRequired,
            overlay: PropTypes.string.isRequired,
            dialog: PropTypes.string.isRequired,
            title: PropTypes.string.isRequired,
            closeButton: PropTypes.string.isRequired,
        }),
        // Dialog content.
        // Anything that can be rendered: numbers, strings, elements or an array
        // (or fragment) containing these types.
        children: PropTypes.node,
    };

    exports.A11yDialog = A11yDialog;
    exports.useA11yDialog = useA11yDialog;

}));
